"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function getUsageStatement(commandName, cliOptions, argumentNames) {
    let statement = `Usage: ${commandName}`;
    let options = '';
    if (cliOptions.length > 0) {
        statement += ` [options...]`;
    }
    if (argumentNames) {
        statement += ` ${argumentNames.join(' ')}`;
    }
    if (cliOptions.length > 0) {
        statement += `\n${getOptionsList(cliOptions)}`;
    }
    return statement;
}
exports.getUsageStatement = getUsageStatement;
function getOptionsList(cliOptions) {
    const descriptions = cliOptions.map(o => {
        let description = `-${o.shortName}`;
        if (o.longName) {
            description += `, --${o.longName}`;
        }
        if (o.argumentName) {
            description += ` ${o.argumentName}`;
        }
        if (o.description) {
            description += ` ${o.description}`;
        }
        return description;
    });
    return descriptions.join('\n');
}
/**
 * A very simple argument parser
 */
class CLIArgumentParser {
    constructor(options) {
        this.optionsByName = new Map();
        options.forEach(option => {
            this.optionsByName.set(option.shortName, option);
            this.optionsByName.set(option.longName, option);
        });
    }
    parseArgs(args) {
        const parsedArguments = {
            options: {},
            arguments: []
        };
        const argumentStartIndex = this.parseOptions(args, parsedArguments);
        parsedArguments.arguments = args.slice(argumentStartIndex);
        return parsedArguments;
    }
    parseOptions(args, parsedArguments) {
        let i = 0;
        let match;
        do {
            let name;
            let value;
            if (match = this.matchesOptionWithValue(args[i])) {
                [name, value] = this.parseOption(match[1], match[2]);
                i++;
            }
            if (!match && (match = this.matchesOption(args[i]))) {
                [name, value] = this.parseOption(match[1], args[i + 1]);
                if (value === undefined) {
                    i++;
                }
                else {
                    i += 2;
                }
            }
            if (!!name) {
                parsedArguments.options[name] = value;
            }
        } while (match && i < args.length);
        return i;
    }
    matchesOptionWithValue(arg) {
        return arg.match(CLIOptionRegExp.shortOptionWithValue) ||
            arg.match(CLIOptionRegExp.longOptionWithValue);
    }
    matchesOption(arg) {
        return arg.match(CLIOptionRegExp.shortOption) ||
            arg.match(CLIOptionRegExp.longOption);
    }
    parseOption(name, value) {
        let nameValue;
        const option = this.optionsByName.get(name);
        if (!option) {
            throw new Error(`Unrecognized option ${name}`);
        }
        nameValue = [option.shortName];
        if (option.takesArgument) {
            if (!value) {
                throw new Error(`Option ${name} requires an argument`);
            }
            nameValue.push(value);
        }
        return nameValue;
    }
}
exports.CLIArgumentParser = CLIArgumentParser;
const CLIOptionRegExp = {
    shortOption: /^-(\w)$/,
    shortOptionWithValue: /^-(\w)=?(\S+)$/,
    longOption: /^--(\S+)$/,
    longOptionWithValue: /^--(\S+)=(\S+)$/,
};
/*export function parseArguments(): CLIArguments {
    if (process.argv.length < 1) {
        throw new Error('Too few arguments');
    }

    const [sourceMap, originalSourceRoot, generatedSourceFile] = process.argv.slice(process.argv.length - 3);

    let stat = fs.statSync(sourceMap);
    if (!stat.isFile()) {
        throw new Error('source_map is not a file');
    }

    stat = fs.statSync(originalSourceRoot);
    if (!stat.isDirectory()) {
        throw new Error('original_source_root is not a directory');
    }

    stat = fs.statSync(generatedSourceFile);
    if (!stat.isFile()) {
        throw new Error('generated_source_file is not a file');
    }

    return {
        sourceMap: sourceMap,
        originalSourceRoot: originalSourceRoot,
        generatedSourceFile: generatedSourceFile,
    }
}

export function parseSourceMapArg(): string {
    const sourceMap = process.argv[0];
    let stat = fs.statSync(process.argv[0]);
    if (!stat.isFile()) {
        throw new Error('source_map is not a file');
    }
    return sourceMap;
}*/
