"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const source_map_1 = require("@lewistg/source-map");
const file_cache_1 = require("./file-cache");
const cli_argument_parser_1 = require("./cli-argument-parser");
const render_html_1 = require("./render-html");
const file_snipper_1 = require("./file-snipper");
function getSnippet(fileContents, lineIndex, columnIndex) {
    const leftPadding = Math.min(Math.max(columnIndex - 1, 0), 10);
    const sourceSnippet = fileContents[lineIndex].slice(columnIndex - leftPadding, columnIndex + 11);
    const snippetMarker = getSpaces(leftPadding) + '^';
    return [sourceSnippet, snippetMarker];
}
function getSpaces(length) {
    let numSpaces = 0;
    let spaces = '';
    while (spaces.length < length) {
        spaces = spaces.length == 0 ? ' ' : spaces + spaces.slice(0, length - spaces.length);
    }
    return spaces;
}
const commandName = 'min-js-annotator';
const cliOptions = [{
        shortName: 'h',
        longName: 'help',
        description: 'Outputs this help text',
    }, {
        shortName: 's',
        longName: 'original-source-root',
        description: 'Overrides "sourceRoot" property in the source map',
        takesArgument: true,
        argumentName: 'DIRECTORY',
    }, {
        shortName: 'm',
        longName: 'source-map',
        description: 'Overrides the file in the minified file\'s "//# sourceMappingURL" comment',
        takesArgument: true,
        argumentName: 'FILE',
    }];
const argumentNames = ['FILE'];
function printUsageAndExit(exitStatus) {
    console.error(cli_argument_parser_1.getUsageStatement(commandName, cliOptions, argumentNames));
    process.exit(exitStatus);
}
function readSourceFile(file) {
    const contents = fs.readFileSync(file, 'utf8');
    return contents.split('\n');
}
function findSourceMapPath(generatedCode) {
    for (let i = generatedCode.length - 1; i >= 0; i--) {
        const match = generatedCode[i].match(sourceMapCommentRegEx);
        if (match && !!match[1]) {
            return match[1];
        }
    }
    return undefined;
}
const sourceMapCommentRegEx = /\/\/#\s*sourceMappingURL=(?:http(?:s?):\/\/.*?\/)?(.*)/;
(function main() {
    const argParser = new cli_argument_parser_1.CLIArgumentParser(cliOptions);
    let cliArguments;
    try {
        cliArguments = argParser.parseArgs(process.argv.slice(2));
    }
    catch (e) {
        printUsageAndExit(1);
    }
    if ('h' in cliArguments.options) {
        printUsageAndExit(0);
    }
    if (cliArguments.arguments.length !== 1) {
        printUsageAndExit(1);
    }
    const generatedFile = cliArguments.arguments[0];
    let generatedCode;
    try {
        generatedCode = readSourceFile(generatedFile);
    }
    catch (e) {
        console.error(`${commandName}: Could not open generated source file ${generatedFile}`);
        process.exit(1);
    }
    const sourceMapFile = cliArguments.options['m'] || findSourceMapPath(generatedCode);
    if (!sourceMapFile) {
        console.error(`${commandName}: Could not find sourceMappingURL in ${generatedFile}. You may provide one using the --source-map option.`);
        process.exit(1);
    }
    let sourceMap;
    try {
        const stringifiedSourceMap = fs.readFileSync(sourceMapFile, 'utf8');
        sourceMap = source_map_1.parseSourceMap(stringifiedSourceMap);
    }
    catch (e) {
        console.error(`${commandName}: Could not read source map ${sourceMapFile}`);
        process.exit(1);
    }
    const sourcesRoot = cliArguments.options['s'] || sourceMap.sourceRoot || path.dirname(sourceMapFile);
    const fileCache = new file_cache_1.FileCache(64);
    const fileSnipper = new file_snipper_1.FileSnipper(sourcesRoot, fileCache);
    console.log(render_html_1.renderAsAnnotatedHtml(generatedCode, sourceMap, fileSnipper));
})();
